@@ -1,9 +1,9 @@
-use ruff_diagnostics::{AlwaysFixableViolation, Diagnostic, Edit, Fix};
-use ruff_macros::{derive_message_formats, violation};
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_python_ast::Stmt;
 use ruff_text_size::Ranged;
 
 use crate::checkers::ast::Checker;
+use crate::{AlwaysFixableViolation, Edit, Fix};
 
 /// ## What it does
 /// Checks for `pass` statements in empty stub bodies.
@@ -13,42 +13,40 @@ use crate::checkers::ast::Checker;
 /// in stub files.
 ///
 /// ## Example
-/// ```python
+/// ```pyi
 /// def foo(bar: int) -> list[int]: pass
 /// ```
 ///
 /// Use instead:
-/// ```python
+/// ```pyi
 /// def foo(bar: int) -> list[int]: ...
 /// ```
 ///
 /// ## References
-/// The [recommended style for functions and methods](https://typing.readthedocs.io/en/latest/source/stubs.html#functions-and-methods)
-/// in the typing docs.
-#[violation]
-pub struct PassStatementStubBody;
+/// - [Typing documentation - Writing and Maintaining Stub Files](https://typing.python.org/en/latest/guides/writing_stubs.html)
+#[derive(ViolationMetadata)]
+pub(crate) struct PassStatementStubBody;
 
 impl AlwaysFixableViolation for PassStatementStubBody {
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("Empty body should contain `...`, not `pass`")
+        "Empty body should contain `...`, not `pass`".to_string()
     }
 
     fn fix_title(&self) -> String {
-        format!("Replace `pass` with `...`")
+        "Replace `pass` with `...`".to_string()
     }
 }
 
 /// PYI009
-pub(crate) fn pass_statement_stub_body(checker: &mut Checker, body: &[Stmt]) {
+pub(crate) fn pass_statement_stub_body(checker: &Checker, body: &[Stmt]) {
     let [Stmt::Pass(pass)] = body else {
         return;
     };
 
-    let mut diagnostic = Diagnostic::new(PassStatementStubBody, pass.range());
+    let mut diagnostic = checker.report_diagnostic(PassStatementStubBody, pass.range());
     diagnostic.set_fix(Fix::safe_edit(Edit::range_replacement(
-        format!("..."),
+        "...".to_string(),
         pass.range(),
     )));
-    checker.diagnostics.push(diagnostic);
 }
