@@ -1,14 +1,12 @@
-use regex::Regex;
 use ruff_python_ast as ast;
-use ruff_python_ast::{Parameter, Parameters};
+use ruff_python_ast::{Parameter, Parameters, Stmt, StmtExpr, StmtFunctionDef, StmtRaise};
 
-use ruff_diagnostics::DiagnosticKind;
-use ruff_diagnostics::{Diagnostic, Violation};
-use ruff_macros::{derive_message_formats, violation};
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_python_semantic::analyze::{function_type, visibility};
 use ruff_python_semantic::{Scope, ScopeKind, SemanticModel};
-use ruff_text_size::Ranged;
+use ruff_text_size::{Ranged, TextRange};
 
+use crate::Violation;
 use crate::checkers::ast::Checker;
 use crate::registry::Rule;
 
@@ -19,6 +17,10 @@ use crate::registry::Rule;
 /// An argument that is defined but not used is likely a mistake, and should
 /// be removed to avoid confusion.
 ///
+/// If a variable is intentionally defined-but-not-used, it should be
+/// prefixed with an underscore, or some other value that adheres to the
+/// [`lint.dummy-variable-rgx`] pattern.
+///
 /// ## Example
 /// ```python
 /// def foo(bar, baz):
@@ -30,8 +32,11 @@ use crate::registry::Rule;
 /// def foo(bar):
 ///     return bar * 2
 /// ```
-#[violation]
-pub struct UnusedFunctionArgument {
+///
+/// ## Options
+/// - `lint.dummy-variable-rgx`
+#[derive(ViolationMetadata)]
+pub(crate) struct UnusedFunctionArgument {
     name: String,
 }
 
@@ -50,6 +55,10 @@ impl Violation for UnusedFunctionArgument {
 /// An argument that is defined but not used is likely a mistake, and should
 /// be removed to avoid confusion.
 ///
+/// If a variable is intentionally defined-but-not-used, it should be
+/// prefixed with an underscore, or some other value that adheres to the
+/// [`lint.dummy-variable-rgx`] pattern.
+///
 /// ## Example
 /// ```python
 /// class Class:
@@ -63,8 +72,11 @@ impl Violation for UnusedFunctionArgument {
 ///     def foo(self, arg1):
 ///         print(arg1)
 /// ```
-#[violation]
-pub struct UnusedMethodArgument {
+///
+/// ## Options
+/// - `lint.dummy-variable-rgx`
+#[derive(ViolationMetadata)]
+pub(crate) struct UnusedMethodArgument {
     name: String,
 }
 
@@ -83,6 +95,10 @@ impl Violation for UnusedMethodArgument {
 /// An argument that is defined but not used is likely a mistake, and should
 /// be removed to avoid confusion.
 ///
+/// If a variable is intentionally defined-but-not-used, it should be
+/// prefixed with an underscore, or some other value that adheres to the
+/// [`lint.dummy-variable-rgx`] pattern.
+///
 /// ## Example
 /// ```python
 /// class Class:
@@ -98,8 +114,11 @@ impl Violation for UnusedMethodArgument {
 ///     def foo(cls, arg1):
 ///         print(arg1)
 /// ```
-#[violation]
-pub struct UnusedClassMethodArgument {
+///
+/// ## Options
+/// - `lint.dummy-variable-rgx`
+#[derive(ViolationMetadata)]
+pub(crate) struct UnusedClassMethodArgument {
     name: String,
 }
 
@@ -118,6 +137,10 @@ impl Violation for UnusedClassMethodArgument {
 /// An argument that is defined but not used is likely a mistake, and should
 /// be removed to avoid confusion.
 ///
+/// If a variable is intentionally defined-but-not-used, it should be
+/// prefixed with an underscore, or some other value that adheres to the
+/// [`lint.dummy-variable-rgx`] pattern.
+///
 /// ## Example
 /// ```python
 /// class Class:
@@ -129,12 +152,15 @@ impl Violation for UnusedClassMethodArgument {
 /// Use instead:
 /// ```python
 /// class Class:
-///     @static
+///     @staticmethod
 ///     def foo(arg1):
 ///         print(arg1)
 /// ```
-#[violation]
-pub struct UnusedStaticMethodArgument {
+///
+/// ## Options
+/// - `lint.dummy-variable-rgx`
+#[derive(ViolationMetadata)]
+pub(crate) struct UnusedStaticMethodArgument {
     name: String,
 }
 
@@ -154,6 +180,10 @@ impl Violation for UnusedStaticMethodArgument {
 /// An argument that is defined but not used is likely a mistake, and should
 /// be removed to avoid confusion.
 ///
+/// If a variable is intentionally defined-but-not-used, it should be
+/// prefixed with an underscore, or some other value that adheres to the
+/// [`lint.dummy-variable-rgx`] pattern.
+///
 /// ## Example
 /// ```python
 /// my_list = [1, 2, 3, 4, 5]
@@ -165,8 +195,11 @@ impl Violation for UnusedStaticMethodArgument {
 /// my_list = [1, 2, 3, 4, 5]
 /// squares = map(lambda x: x**2, my_list)
 /// ```
-#[violation]
-pub struct UnusedLambdaArgument {
+///
+/// ## Options
+/// - `lint.dummy-variable-rgx`
+#[derive(ViolationMetadata)]
+pub(crate) struct UnusedLambdaArgument {
     name: String,
 }
 
@@ -189,14 +222,19 @@ enum Argumentable {
 }
 
 impl Argumentable {
-    fn check_for(self, name: String) -> DiagnosticKind {
-        match self {
-            Self::Function => UnusedFunctionArgument { name }.into(),
-            Self::Method => UnusedMethodArgument { name }.into(),
-            Self::ClassMethod => UnusedClassMethodArgument { name }.into(),
-            Self::StaticMethod => UnusedStaticMethodArgument { name }.into(),
-            Self::Lambda => UnusedLambdaArgument { name }.into(),
-        }
+    fn check_for(self, checker: &Checker, name: String, range: TextRange) {
+        let mut diagnostic = match self {
+            Self::Function => checker.report_diagnostic(UnusedFunctionArgument { name }, range),
+            Self::Method => checker.report_diagnostic(UnusedMethodArgument { name }, range),
+            Self::ClassMethod => {
+                checker.report_diagnostic(UnusedClassMethodArgument { name }, range)
+            }
+            Self::StaticMethod => {
+                checker.report_diagnostic(UnusedStaticMethodArgument { name }, range)
+            }
+            Self::Lambda => checker.report_diagnostic(UnusedLambdaArgument { name }, range),
+        };
+        diagnostic.add_primary_tag(ruff_db::diagnostic::DiagnosticTag::Unnecessary);
     }
 
     const fn rule_code(self) -> Rule {
@@ -211,15 +249,11 @@ impl Argumentable {
 }
 
 /// Check a plain function for unused arguments.
-fn function(
-    argumentable: Argumentable,
-    parameters: &Parameters,
-    scope: &Scope,
-    semantic: &SemanticModel,
-    dummy_variable_rgx: &Regex,
-    ignore_variadic_names: bool,
-    diagnostics: &mut Vec<Diagnostic>,
-) {
+fn function(argumentable: Argumentable, parameters: &Parameters, scope: &Scope, checker: &Checker) {
+    let ignore_variadic_names = checker
+        .settings()
+        .flake8_unused_arguments
+        .ignore_variadic_names;
     let args = parameters
         .iter_non_variadic_params()
         .map(|parameter_with_default| &parameter_with_default.parameter)
@@ -237,26 +271,15 @@ fn function(
                 .into_iter()
                 .skip(usize::from(ignore_variadic_names)),
         );
-    call(
-        argumentable,
-        args,
-        scope,
-        semantic,
-        dummy_variable_rgx,
-        diagnostics,
-    );
+    call(argumentable, args, scope, checker);
 }
 
 /// Check a method for unused arguments.
-fn method(
-    argumentable: Argumentable,
-    parameters: &Parameters,
-    scope: &Scope,
-    semantic: &SemanticModel,
-    dummy_variable_rgx: &Regex,
-    ignore_variadic_names: bool,
-    diagnostics: &mut Vec<Diagnostic>,
-) {
+fn method(argumentable: Argumentable, parameters: &Parameters, scope: &Scope, checker: &Checker) {
+    let ignore_variadic_names = checker
+        .settings()
+        .flake8_unused_arguments
+        .ignore_variadic_names;
     let args = parameters
         .iter_non_variadic_params()
         .skip(1)
@@ -275,48 +298,95 @@ fn method(
                 .into_iter()
                 .skip(usize::from(ignore_variadic_names)),
         );
-    call(
-        argumentable,
-        args,
-        scope,
-        semantic,
-        dummy_variable_rgx,
-        diagnostics,
-    );
+    call(argumentable, args, scope, checker);
 }
 
 fn call<'a>(
     argumentable: Argumentable,
     parameters: impl Iterator<Item = &'a Parameter>,
     scope: &Scope,
-    semantic: &SemanticModel,
-    dummy_variable_rgx: &Regex,
-    diagnostics: &mut Vec<Diagnostic>,
+    checker: &Checker,
 ) {
-    diagnostics.extend(parameters.filter_map(|arg| {
-        let binding = scope
-            .get(arg.name.as_str())
-            .map(|binding_id| semantic.binding(binding_id))?;
+    let semantic = checker.semantic();
+    let dummy_variable_rgx = &checker.settings().dummy_variable_rgx;
+    for arg in parameters {
+        let Some(binding) = scope
+            .get(arg.name())
+            .map(|binding_id| semantic.binding(binding_id))
+        else {
+            continue;
+        };
         if binding.kind.is_argument()
-            && !binding.is_used()
-            && !dummy_variable_rgx.is_match(arg.name.as_str())
+            && binding.is_unused()
+            && !dummy_variable_rgx.is_match(arg.name())
         {
-            Some(Diagnostic::new(
-                argumentable.check_for(arg.name.to_string()),
-                binding.range(),
-            ))
-        } else {
-            None
+            argumentable.check_for(checker, arg.name.to_string(), binding.range());
         }
-    }));
+    }
+}
+
+/// Returns `true` if a function appears to be a base class stub. In other
+/// words, if it matches the following syntax:
+///
+/// ```text
+/// variable = <string | f-string>
+/// raise NotImplementedError(variable)
+/// ```
+///
+/// See also [`is_stub`]. We're a bit more generous in what is considered a
+/// stub in this rule to avoid clashing with [`EM101`].
+///
+/// [`is_stub`]: function_type::is_stub
+/// [`EM101`]: https://docs.astral.sh/ruff/rules/raw-string-in-exception/
+pub(crate) fn is_not_implemented_stub_with_variable(
+    function_def: &StmtFunctionDef,
+    semantic: &SemanticModel,
+) -> bool {
+    // Ignore doc-strings.
+    let statements = match function_def.body.as_slice() {
+        [Stmt::Expr(StmtExpr { value, .. }), rest @ ..] if value.is_string_literal_expr() => rest,
+        _ => &function_def.body,
+    };
+
+    let [
+        Stmt::Assign(ast::StmtAssign { targets, value, .. }),
+        Stmt::Raise(StmtRaise {
+            exc: Some(exception),
+            ..
+        }),
+    ] = statements
+    else {
+        return false;
+    };
+
+    if !matches!(**value, ast::Expr::StringLiteral(_) | ast::Expr::FString(_)) {
+        return false;
+    }
+
+    let ast::Expr::Call(ast::ExprCall {
+        func, arguments, ..
+    }) = &**exception
+    else {
+        return false;
+    };
+
+    if !semantic.match_builtin_expr(func, "NotImplementedError") {
+        return false;
+    }
+
+    let [argument] = &*arguments.args else {
+        return false;
+    };
+
+    let [target] = targets.as_slice() else {
+        return false;
+    };
+
+    argument.as_name_expr().map(ast::ExprName::id) == target.as_name_expr().map(ast::ExprName::id)
 }
 
 /// ARG001, ARG002, ARG003, ARG004, ARG005
-pub(crate) fn unused_arguments(
-    checker: &Checker,
-    scope: &Scope,
-    diagnostics: &mut Vec<Diagnostic>,
-) {
+pub(crate) fn unused_arguments(checker: &Checker, scope: &Scope) {
     if scope.uses_locals() {
         return;
     }
@@ -339,119 +409,79 @@ pub(crate) fn unused_arguments(
                 decorator_list,
                 parent,
                 checker.semantic(),
-                &checker.settings.pep8_naming.classmethod_decorators,
-                &checker.settings.pep8_naming.staticmethod_decorators,
+                &checker.settings().pep8_naming.classmethod_decorators,
+                &checker.settings().pep8_naming.staticmethod_decorators,
             ) {
                 function_type::FunctionType::Function => {
-                    if checker.enabled(Argumentable::Function.rule_code())
+                    if checker.is_rule_enabled(Argumentable::Function.rule_code())
+                        && !function_type::is_stub(function_def, checker.semantic())
+                        && !is_not_implemented_stub_with_variable(function_def, checker.semantic())
                         && !visibility::is_overload(decorator_list, checker.semantic())
                     {
-                        function(
-                            Argumentable::Function,
-                            parameters,
-                            scope,
-                            checker.semantic(),
-                            &checker.settings.dummy_variable_rgx,
-                            checker
-                                .settings
-                                .flake8_unused_arguments
-                                .ignore_variadic_names,
-                            diagnostics,
-                        );
+                        function(Argumentable::Function, parameters, scope, checker);
                     }
                 }
                 function_type::FunctionType::Method => {
-                    if checker.enabled(Argumentable::Method.rule_code())
+                    if checker.is_rule_enabled(Argumentable::Method.rule_code())
                         && !function_type::is_stub(function_def, checker.semantic())
+                        && !is_not_implemented_stub_with_variable(function_def, checker.semantic())
                         && (!visibility::is_magic(name)
                             || visibility::is_init(name)
-                            || visibility::is_new(name)
                             || visibility::is_call(name))
                         && !visibility::is_abstract(decorator_list, checker.semantic())
                         && !visibility::is_override(decorator_list, checker.semantic())
                         && !visibility::is_overload(decorator_list, checker.semantic())
                     {
-                        method(
-                            Argumentable::Method,
-                            parameters,
-                            scope,
-                            checker.semantic(),
-                            &checker.settings.dummy_variable_rgx,
-                            checker
-                                .settings
-                                .flake8_unused_arguments
-                                .ignore_variadic_names,
-                            diagnostics,
-                        );
+                        method(Argumentable::Method, parameters, scope, checker);
                     }
                 }
                 function_type::FunctionType::ClassMethod => {
-                    if checker.enabled(Argumentable::ClassMethod.rule_code())
+                    if checker.is_rule_enabled(Argumentable::ClassMethod.rule_code())
                         && !function_type::is_stub(function_def, checker.semantic())
+                        && !is_not_implemented_stub_with_variable(function_def, checker.semantic())
                         && (!visibility::is_magic(name)
                             || visibility::is_init(name)
-                            || visibility::is_new(name)
                             || visibility::is_call(name))
                         && !visibility::is_abstract(decorator_list, checker.semantic())
                         && !visibility::is_override(decorator_list, checker.semantic())
                         && !visibility::is_overload(decorator_list, checker.semantic())
                     {
-                        method(
-                            Argumentable::ClassMethod,
-                            parameters,
-                            scope,
-                            checker.semantic(),
-                            &checker.settings.dummy_variable_rgx,
-                            checker
-                                .settings
-                                .flake8_unused_arguments
-                                .ignore_variadic_names,
-                            diagnostics,
-                        );
+                        method(Argumentable::ClassMethod, parameters, scope, checker);
                     }
                 }
                 function_type::FunctionType::StaticMethod => {
-                    if checker.enabled(Argumentable::StaticMethod.rule_code())
+                    if checker.is_rule_enabled(Argumentable::StaticMethod.rule_code())
                         && !function_type::is_stub(function_def, checker.semantic())
+                        && !is_not_implemented_stub_with_variable(function_def, checker.semantic())
                         && (!visibility::is_magic(name)
                             || visibility::is_init(name)
-                            || visibility::is_new(name)
                             || visibility::is_call(name))
                         && !visibility::is_abstract(decorator_list, checker.semantic())
                         && !visibility::is_override(decorator_list, checker.semantic())
                         && !visibility::is_overload(decorator_list, checker.semantic())
                     {
-                        function(
-                            Argumentable::StaticMethod,
-                            parameters,
-                            scope,
-                            checker.semantic(),
-                            &checker.settings.dummy_variable_rgx,
-                            checker
-                                .settings
-                                .flake8_unused_arguments
-                                .ignore_variadic_names,
-                            diagnostics,
-                        );
+                        function(Argumentable::StaticMethod, parameters, scope, checker);
+                    }
+                }
+                function_type::FunctionType::NewMethod => {
+                    if checker.is_rule_enabled(Argumentable::StaticMethod.rule_code())
+                        && !function_type::is_stub(function_def, checker.semantic())
+                        && !is_not_implemented_stub_with_variable(function_def, checker.semantic())
+                        && !visibility::is_abstract(decorator_list, checker.semantic())
+                        && !visibility::is_override(decorator_list, checker.semantic())
+                        && !visibility::is_overload(decorator_list, checker.semantic())
+                    {
+                        // we use `method()` here rather than `function()`, as although `__new__` is
+                        // an implicit staticmethod, `__new__` methods must always have at least one parameter
+                        method(Argumentable::StaticMethod, parameters, scope, checker);
                     }
                 }
             }
         }
         ScopeKind::Lambda(ast::ExprLambda { parameters, .. }) => {
             if let Some(parameters) = parameters {
-                if checker.enabled(Argumentable::Lambda.rule_code()) {
-                    function(
-                        Argumentable::Lambda,
-                        parameters,
-                        scope,
-                        checker.semantic(),
-                        &checker.settings.dummy_variable_rgx,
-                        checker
-                            .settings
-                            .flake8_unused_arguments
-                            .ignore_variadic_names,
-                        diagnostics,
-                    );
+                if checker.is_rule_enabled(Argumentable::Lambda.rule_code()) {
+                    function(Argumentable::Lambda, parameters, scope, checker);
                 }
             }
         }
