@@ -1,10 +1,10 @@
-use ruff_diagnostics::{Diagnostic, Violation};
-use ruff_macros::{derive_message_formats, violation};
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_python_ast as ast;
-use ruff_python_semantic::analyze::type_inference::{PythonType, ResolvedPythonType};
 use ruff_python_semantic::Modules;
+use ruff_python_semantic::analyze::type_inference::{PythonType, ResolvedPythonType};
 use ruff_text_size::Ranged;
 
+use crate::Violation;
 use crate::checkers::ast::Checker;
 
 /// ## What it does
@@ -32,18 +32,18 @@ use crate::checkers::ast::Checker;
 ///
 /// int(os.getenv("FOO", "1"))
 /// ```
-#[violation]
-pub struct InvalidEnvvarDefault;
+#[derive(ViolationMetadata)]
+pub(crate) struct InvalidEnvvarDefault;
 
 impl Violation for InvalidEnvvarDefault {
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("Invalid type for environment variable default; expected `str` or `None`")
+        "Invalid type for environment variable default; expected `str` or `None`".to_string()
     }
 }
 
 /// PLW1508
-pub(crate) fn invalid_envvar_default(checker: &mut Checker, call: &ast::ExprCall) {
+pub(crate) fn invalid_envvar_default(checker: &Checker, call: &ast::ExprCall) {
     if !checker.semantic().seen_module(Modules::OS) {
         return;
     }
@@ -51,10 +51,15 @@ pub(crate) fn invalid_envvar_default(checker: &mut Checker, call: &ast::ExprCall
     if checker
         .semantic()
         .resolve_qualified_name(&call.func)
-        .is_some_and(|qualified_name| matches!(qualified_name.segments(), ["os", "getenv"]))
+        .is_some_and(|qualified_name| {
+            matches!(
+                qualified_name.segments(),
+                ["os", "getenv"] | ["os", "environ", "get"]
+            )
+        })
     {
         // Find the `default` argument, if it exists.
-        let Some(expr) = call.arguments.find_argument("default", 1) else {
+        let Some(expr) = call.arguments.find_argument_value("default", 1) else {
             return;
         };
 
@@ -65,8 +70,6 @@ pub(crate) fn invalid_envvar_default(checker: &mut Checker, call: &ast::ExprCall
         ) {
             return;
         }
-        checker
-            .diagnostics
-            .push(Diagnostic::new(InvalidEnvvarDefault, expr.range()));
+        checker.report_diagnostic(InvalidEnvvarDefault, expr.range());
     }
 }
