@@ -1,4 +1,4 @@
-use ruff_formatter::{write, Argument, Arguments, FormatError};
+use ruff_formatter::{Argument, Arguments, FormatError, write};
 use ruff_python_ast::AnyNodeRef;
 use ruff_python_ast::{
     ElifElseClause, ExceptHandlerExceptHandler, MatchCase, StmtClassDef, StmtFor, StmtFunctionDef,
@@ -7,8 +7,8 @@ use ruff_python_ast::{
 use ruff_python_trivia::{SimpleToken, SimpleTokenKind, SimpleTokenizer};
 use ruff_text_size::{Ranged, TextRange, TextSize};
 
-use crate::comments::{leading_alternate_branch_comments, trailing_comments, SourceComment};
-use crate::statement::suite::{contains_only_an_ellipsis, SuiteKind};
+use crate::comments::{SourceComment, leading_alternate_branch_comments, trailing_comments};
+use crate::statement::suite::{SuiteKind, contains_only_an_ellipsis};
 use crate::verbatim::write_suppressed_clause_header;
 use crate::{has_skip_comment, prelude::*};
 
@@ -17,6 +17,7 @@ use crate::{has_skip_comment, prelude::*};
 /// > A compound statement consists of one or more ‘clauses.’ A clause consists of a header and a ‘suite.’
 /// > The clause headers of a particular compound statement are all at the same indentation level.
 /// > Each clause header begins with a uniquely identifying keyword and ends with a colon.
+///
 /// [source](https://docs.python.org/3/reference/compound_stmts.html#compound-statements)
 #[derive(Copy, Clone)]
 pub(crate) enum ClauseHeader<'a> {
@@ -35,7 +36,7 @@ pub(crate) enum ClauseHeader<'a> {
     OrElse(ElseClause<'a>),
 }
 
-impl<'a> ClauseHeader<'a> {
+impl ClauseHeader<'_> {
     /// The range from the clause keyword up to and including the final colon.
     pub(crate) fn range(self, source: &str) -> FormatResult<TextRange> {
         let keyword_range = self.first_keyword_range(source)?;
@@ -86,6 +87,7 @@ impl<'a> ClauseHeader<'a> {
                 type_params,
                 arguments,
                 range: _,
+                node_index: _,
                 decorator_list: _,
                 name: _,
                 body: _,
@@ -102,13 +104,14 @@ impl<'a> ClauseHeader<'a> {
                 type_params,
                 parameters,
                 range: _,
+                node_index: _,
                 is_async: _,
                 decorator_list: _,
                 name: _,
                 returns,
                 body: _,
             }) => {
-                if let Some(type_params) = type_params.as_ref() {
+                if let Some(type_params) = type_params.as_deref() {
                     visit(type_params, visitor);
                 }
                 visit(parameters.as_ref(), visitor);
@@ -120,6 +123,7 @@ impl<'a> ClauseHeader<'a> {
             ClauseHeader::If(StmtIf {
                 test,
                 range: _,
+                node_index: _,
                 body: _,
                 elif_else_clauses: _,
             }) => {
@@ -128,6 +132,7 @@ impl<'a> ClauseHeader<'a> {
             ClauseHeader::ElifElse(ElifElseClause {
                 test,
                 range: _,
+                node_index: _,
                 body: _,
             }) => {
                 if let Some(test) = test.as_ref() {
@@ -138,6 +143,7 @@ impl<'a> ClauseHeader<'a> {
             ClauseHeader::ExceptHandler(ExceptHandlerExceptHandler {
                 type_: type_expr,
                 range: _,
+                node_index: _,
                 name: _,
                 body: _,
             }) => {
@@ -148,6 +154,7 @@ impl<'a> ClauseHeader<'a> {
             ClauseHeader::Match(StmtMatch {
                 subject,
                 range: _,
+                node_index: _,
                 cases: _,
             }) => {
                 visit(subject.as_ref(), visitor);
@@ -156,6 +163,7 @@ impl<'a> ClauseHeader<'a> {
                 guard,
                 pattern,
                 range: _,
+                node_index: _,
                 body: _,
             }) => {
                 visit(pattern, visitor);
@@ -168,6 +176,7 @@ impl<'a> ClauseHeader<'a> {
                 target,
                 iter,
                 range: _,
+                node_index: _,
                 is_async: _,
                 body: _,
                 orelse: _,
@@ -178,6 +187,7 @@ impl<'a> ClauseHeader<'a> {
             ClauseHeader::While(StmtWhile {
                 test,
                 range: _,
+                node_index: _,
                 body: _,
                 orelse: _,
             }) => {
@@ -186,6 +196,7 @@ impl<'a> ClauseHeader<'a> {
             ClauseHeader::With(StmtWith {
                 items,
                 range: _,
+                node_index: _,
                 is_async: _,
                 body: _,
             }) => {
@@ -380,21 +391,14 @@ pub(crate) struct FormatClauseBody<'a> {
     trailing_comments: &'a [SourceComment],
 }
 
-impl<'a> FormatClauseBody<'a> {
-    #[must_use]
-    pub(crate) fn with_kind(mut self, kind: SuiteKind) -> Self {
-        self.kind = kind;
-        self
-    }
-}
-
 pub(crate) fn clause_body<'a>(
     body: &'a Suite,
+    kind: SuiteKind,
     trailing_comments: &'a [SourceComment],
 ) -> FormatClauseBody<'a> {
     FormatClauseBody {
         body,
-        kind: SuiteKind::default(),
+        kind,
         trailing_comments,
     }
 }
@@ -479,12 +483,22 @@ fn colon_range(after_keyword_or_condition: TextSize, source: &str) -> FormatResu
             range,
         }) => Ok(range),
         Some(token) => {
-            debug_assert!(false, "Expected the colon marking the end of the case header but found {token:?} instead.");
-            Err(FormatError::syntax_error("Expected colon marking the end of the case header but found another token instead."))
+            debug_assert!(
+                false,
+                "Expected the colon marking the end of the case header but found {token:?} instead."
+            );
+            Err(FormatError::syntax_error(
+                "Expected colon marking the end of the case header but found another token instead.",
+            ))
         }
         None => {
-            debug_assert!(false, "Expected the colon marking the end of the case header but found the end of the range.");
-            Err(FormatError::syntax_error("Expected the colon marking the end of the case header but found the end of the range."))
+            debug_assert!(
+                false,
+                "Expected the colon marking the end of the case header but found the end of the range."
+            );
+            Err(FormatError::syntax_error(
+                "Expected the colon marking the end of the case header but found the end of the range.",
+            ))
         }
     }
 }
