@@ -1,11 +1,13 @@
 use super::context::InferContext;
 use super::{Signature, Type};
 use crate::Db;
+use crate::types::PropertyInstanceType;
+use crate::types::call::bind::BindingError;
 
 mod arguments;
 pub(crate) mod bind;
-pub(super) use arguments::{Argument, CallArgumentTypes, CallArguments};
-pub(super) use bind::{Binding, Bindings, CallableBinding};
+pub(super) use arguments::{Argument, CallArguments};
+pub(super) use bind::{Binding, Bindings, CallableBinding, MatchedArgument};
 
 /// Wraps a [`Bindings`] for an unsuccessful call with information about why the call was
 /// unsuccessful.
@@ -14,6 +16,26 @@ pub(super) use bind::{Binding, Bindings, CallableBinding};
 #[derive(Debug)]
 pub(crate) struct CallError<'db>(pub(crate) CallErrorKind, pub(crate) Box<Bindings<'db>>);
 
+impl<'db> CallError<'db> {
+    /// Returns `Some(property)` if the call error was caused by an attempt to set a property
+    /// that has no setter, and `None` otherwise.
+    pub(crate) fn as_attempt_to_set_property_with_no_setter(
+        &self,
+    ) -> Option<PropertyInstanceType<'db>> {
+        if self.0 != CallErrorKind::BindingError {
+            return None;
+        }
+        self.1
+            .into_iter()
+            .flatten()
+            .flat_map(bind::Binding::errors)
+            .find_map(|error| match error {
+                BindingError::PropertyHasNoSetter(property) => Some(*property),
+                _ => None,
+            })
+    }
+}
+
 /// The reason why calling a type failed.
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub(crate) enum CallErrorKind {
